# SPDX-FileCopyrightText: 2025 Lubin Bailly <lubin.bailly@dgnum.eu>
#
# SPDX-License-Identifier: EUPL-1.2

{ lib, config, ... }:
let
  inherit (lib)
    mkMerge
    listToAttrs
    nameValuePair
    ;
  branches = [
    "master"
    "gestion-clubs"
    "status-change-2024-nov-13"
  ];
  mainDomain = "cof-stage.dgnum.eu";
in
mkMerge (
  [
    {
      services.django-apps = {
        enable = true;

        webhook = {
          domain = mainDomain;

          nginx = {
            enableACME = true;
            forceSSL = true;
          };
        };
      };
    }
  ]
  ++ map (branch: {
    services.django-apps.sites.${branch} = {
      inherit branch;
      source = "https://git.dgnum.eu/DGNum/gestioCOF";
      domain = "${branch}.${mainDomain}";
      doBackups = false;

      nginx = {
        enableACME = true;
        forceSSL = true;

        locations =
          {
            "/ws/" = {
              proxyPass = "http://unix:/run/django-apps/${branch}/socket";
              proxyWebsockets = true;
            };
          }
          // (listToAttrs (
            builtins.map
              (
                folder:
                nameValuePair "~ ^/${folder}/" {
                  extraConfig = ''
                    # we don't have the mount
                    return 501;
                  '';
                }
              )
              [
                "arcanoid"
                "cirque"
                "pompom"
                "trouveres"
              ]
          ));

        extraConfig = ''
          rewrite ^/$ /news;
        '';
      };

      webHookSecret = config.age.secrets."webhook-gestiocof_token".path;

      overlays = {
        kat-pkgs = [
          "django-djconfig"
          "django-hCaptcha"
          "wagtail-modeltranslation"
          "wagtailmenus"
          "django-cogwheels"
        ];

        nix-pkgs = [
          # Required packages
          "authens"
          "django-bootstrap-form"
          "django-cas-ng"
          "loadcredential"

          # Dependencies
          "python-cas"
        ];
      };

      dependencies = ps: [
        ps.authens
        ps.channels
        ps.channels-redis
        ps.configparser
        ps.django
        ps.django-autocomplete-light
        ps.django-bootstrap-form
        ps.django-cas-ng
        ps.django-cors-headers
        ps.django-djconfig
        ps.django-hCaptcha
        ps.django-js-reverse
        ps.django-widget-tweaks
        ps.icalendar
        ps.loadcredential
        ps.pillow
        ps.python-dateutil
        ps.redis
        ps.statistics
        ps.wagtail
        ps.wagtail-modeltranslation
        ps.wagtailmenus
      ];

      application = {
        module = "gestioasso";
        type = "daphne";
        settingsModule = "gestioasso.settings_cof";
      };

      environment = {
        GESTIOCOF_CHANNEL_LAYERS.default = {
          BACKEND = "shared.channels.ChannelLayer";
          CONFIG.hosts = [ "unix://${config.services.redis.servers.${branch}.unixSocket}" ];
        };
        GESTIOCOF_CACHES.default = {
          BACKEND = "django.core.cache.backends.redis.RedisCache";
          LOCATION = "unix://${config.services.redis.servers.${branch}.unixSocket}";
        };
        GESTIOCOF_CORS_ALLOWED_ORIGINS = [
          "https://${config.services.django-apps.sites.${branch}.domain}"
        ];
        GESTIOCOF_SERVER_EMAIL = "gestion@cof.ens.fr";

        # fake credentials
        GESTIOCOF_SECRET_KEY = "insecure-key";
        GESTIOCOF_HCAPTCHA_SECRET = "0x0000000000000000000000000000000000000000";
        GESTIOCOF_HCAPTCHA_SITEKEY = "10000000-ffff-ffff-ffff-000000000001";
        GESTIOCOF_KFETOPEN_TOKEN = "k-feste_token";
        GESTIOCOF_SYMPA_PASSWORD = "toto";
        GESTIOCOF_SYMPA_USERNAME = "sympa";
        GESTIOCOF_EMAIL_HOST = "localhost";
      };
      env_prefix = "GESTIOCOF";

      extraInit = ''
        python manage.py sync_page_translation_fields
        python manage.py update_translation_fields
        python manage.py loaddata gestion sites articles
        python manage.py loaddevdata
      '';

      extraServices.worker = {
        script = "python3 manage.py runworker default";

        serviceConfig = {
          WorkingDirectory = "/var/lib/django-apps/${branch}/source";
          SupplementaryGroups = [ "redis-${branch}" ];
        };
      };
      timers = {
        rappel-negatifs = {
          script = ''
            python3 manage.py sendrappelsnegatifs
          '';
          startAt = "*-*-* 1,13:17:19";
        };
        rappel-bda = {
          script = ''
            python3 manage.py sendrappels
          '';
          startAt = "*-*-* 2,14:17:19";
        };
        manage-reventes = {
          script = ''
            python3 manage.py manage_reventes
          '';
          startAt = "*-*-* *:01..56/5:29";
        };
      };
    };

    services.redis.servers.${branch} = {
      enable = true;
    };

    systemd.services."dj-${branch}".serviceConfig.SupplementaryGroups = [ "redis-${branch}" ];
  }) branches
)
